@@ -37,24 +37,28 @@
 #include <dbus/dbus-glib-bindings.h>
 
 #define HEADSET_CONTROL_TYPE_2_STRINGS G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID
+#define HAL_DEVICE_ADDED "DeviceAdded"
 #define HAL_CONDITION "Condition"
 #define BUTTON_PRESSED_CONDITION "ButtonPressed"
 #define CONDITION_DETAIL_PHONE "phone"
 #define HAL_SERVICE "org.freedesktop.Hal"
 #define HEADSET_HAL_OBJ_PATH "/org/freedesktop/Hal/devices/computer_logicaldev_input_1"
 #define HAL_DEVICE_INTERFACE "org.freedesktop.Hal.Device"
+#define HAL_MANAGER_PATH "/org/freedesktop/Hal/Manager"
+#define HAL_MANAGER_INTERFACE "org.freedesktop.Hal.Manager"
 
 #define HEADSET_CONTROL_NEXT_MODE_KEY HEADSET_CONTROL_GCONF_FOLDER NEXT_MODE_GCONF_KEY
 #define WANTED_RENDERER "Mafw-Gst-Renderer"
 
-#define IS_IN_CALL g_str_equal (call_state, MCE_CALL_STATE_RINGING) || g_str_equal (call_state, MCE_CALL_STATE_ACTIVE)
+#define IS_IN_CALL(call_state) g_str_equal ((call_state), MCE_CALL_STATE_ACTIVE) || g_str_equal ((call_state), MCE_CALL_STATE_RINGING)
 
 #define HEADSET_CONTROL_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
 						HEADSET_TYPE_CONTROL, HeadsetControlPrivate))
 
 struct _HeadsetControlPrivate
 {
-	DBusGProxy *hal_proxy;
+	DBusGProxy *hal_device_proxy;
+	DBusGProxy *hal_manager_proxy;
 	MafwRenderer *gst_renderer;
 	DBusGConnection *sess_dbus_conn;
 	pb_playback_t *playback;
@@ -91,6 +95,7 @@ static void headset_control_renderer_removed (MafwRegistry *, MafwRenderer *, He
 static void headset_control_renderer_added_cb (MafwRegistry *, GObject *, HeadsetControl *);
 static gboolean headset_control_suspend_off (HeadsetControl *);
 static void headset_control_stop_suspend_off_timer (HeadsetControl *);
+static void headset_control_device_added (DBusGProxy *, const gchar *, HeadsetControl *);
 
 /* -- */
 
@@ -121,7 +126,6 @@ static void headset_control_get_property (GObject *object, guint property_id, GV
 static void headset_control_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec * pspec)
 {
 	HeadsetControl *self = HEADSET_CONTROL (object);
-	HeadsetControlClass *klass = HEADSET_CONTROL_GET_CLASS (self);
  	  	 
 	switch (property_id)
 	{
@@ -162,7 +166,12 @@ static void headset_control_finalize (GObject *object)
 
 	headset_control_stop_suspend_off_timer (self);
 
-	self->priv->state = _LastMafwPlayState;
+	if (self->priv->hal_manager_proxy)
+	{
+		dbus_g_proxy_disconnect_signal (self->priv->hal_manager_proxy, HAL_DEVICE_ADDED, G_CALLBACK (headset_control_device_added), self);
+		g_object_unref (G_OBJECT (self->priv->hal_manager_proxy));
+		self->priv->hal_manager_proxy = NULL;
+	}
 
 	if (self->priv->mce_signal_proxy)
 	{
@@ -170,12 +179,12 @@ static void headset_control_finalize (GObject *object)
 		g_object_unref (G_OBJECT (self->priv->mce_signal_proxy));
 		self->priv->mce_signal_proxy = NULL;
 	}
-
-	if (self->priv->hal_proxy)
+		
+	if (self->priv->hal_device_proxy)
 	{
-		dbus_g_proxy_disconnect_signal (self->priv->hal_proxy, HAL_CONDITION, G_CALLBACK (headset_control_button_pressed), self);
-		g_object_unref (G_OBJECT (self->priv->hal_proxy));
-		self->priv->hal_proxy = NULL;
+		dbus_g_proxy_disconnect_signal (self->priv->hal_device_proxy, HAL_CONDITION, G_CALLBACK (headset_control_button_pressed), self);
+		g_object_unref (G_OBJECT (self->priv->hal_device_proxy));
+		self->priv->hal_device_proxy = NULL;
 	}
 
 	g_signal_handlers_disconnect_by_func (klass->mafw_registry, G_CALLBACK (headset_control_renderer_added_cb), self); /* Don't really desire much for renderers to be added... */
@@ -218,8 +227,6 @@ static void headset_control_finalize (GObject *object)
 		klass->sys_dbus_conn = NULL;
 	}
 	
-	self->priv->state = _LastMafwPlayState;
-
 	if (G_OBJECT_CLASS (headset_control_parent_class)->finalize)
 		G_OBJECT_CLASS (headset_control_parent_class)->finalize (object);
 }
@@ -352,12 +359,9 @@ static void headset_control_rend_command_sent (MafwRenderer *renderer G_GNUC_UNU
 	pb_playback_req_state (self->priv->playback, self->priv->new_pb_state, (PBStateReply) headset_control_libplayback_state_reply, NULL);
 }
 
-static void headset_control_button_pressed (DBusGProxy *hal_proxy G_GNUC_UNUSED, const gchar *condition_name, const gchar *condition_detail, HeadsetControl *self)
+static void headset_control_button_pressed (DBusGProxy *hal_device_proxy G_GNUC_UNUSED, const gchar *condition_name, const gchar *condition_detail, HeadsetControl *self)
 {
-	if (!g_str_equal (condition_name, BUTTON_PRESSED_CONDITION) || !g_str_equal (condition_detail, CONDITION_DETAIL_PHONE))
-		return;
-
-	if (self->priv->in_call)
+	if (!g_str_equal (condition_detail, CONDITION_DETAIL_PHONE) || !g_str_equal (condition_name, BUTTON_PRESSED_CONDITION) || self->priv->in_call)
 		return;
 
 	if (self->priv->suspended)
@@ -396,7 +400,7 @@ static void headset_control_button_pressed (DBusGProxy *hal_proxy G_GNUC_UNUSED,
 
 static void headset_control_state_changed (MafwRenderer *renderer G_GNUC_UNUSED, MafwPlayState state, HeadsetControl *self)
 {
-        self->priv->state = state;
+	self->priv->state = state;
 }
 
 static void headset_control_status_recieved (MafwRenderer *renderer, MafwPlaylist *playlist G_GNUC_UNUSED, guint index G_GNUC_UNUSED, MafwPlayState state, const gchar *object_id G_GNUC_UNUSED, HeadsetControl *self, const GError *error)
@@ -447,12 +451,20 @@ static void headset_control_next_mode_key_changed (GConfClient *gconf_client G_G
 
 static void headset_control_call_state_changed (DBusGProxy *mce_proxy G_GNUC_UNUSED, const gchar *call_state, const gchar *emergency_call_state G_GNUC_UNUSED, HeadsetControl *self)
 {
-	self->priv->in_call = IS_IN_CALL;
+	self->priv->in_call = IS_IN_CALL (call_state);
 
 	if (!self->priv->in_call)
 		g_timeout_add_seconds (5, (GSourceFunc) headset_control_enable_headset_button, NULL); /* Stupidest fucking hack ever */
 }
 
+static void headset_control_device_added (DBusGProxy *hal_manager_proxy G_GNUC_UNUSED, const gchar *device, HeadsetControl *self)
+{
+	if (!g_str_equal (device, HEADSET_HAL_OBJ_PATH) || self->priv->in_call)
+		return;
+
+	g_timeout_add_seconds (5, (GSourceFunc) headset_control_enable_headset_button, NULL);
+}
+
 /* -- */
 
 static void headset_control_constructed (GObject *object)
@@ -462,7 +474,8 @@ static void headset_control_constructed (GObject *object)
 	if (G_OBJECT_CLASS (headset_control_parent_class)->constructed)
 		G_OBJECT_CLASS (headset_control_parent_class)->constructed (object);
 
-	dbus_g_proxy_connect_signal (self->priv->hal_proxy, HAL_CONDITION, G_CALLBACK (headset_control_button_pressed), self, NULL);
+	dbus_g_proxy_connect_signal (self->priv->hal_device_proxy, HAL_CONDITION, G_CALLBACK (headset_control_button_pressed), self, NULL);
+	dbus_g_proxy_connect_signal (self->priv->hal_manager_proxy, HAL_DEVICE_ADDED, G_CALLBACK (headset_control_device_added), self, NULL);
 	(void) headset_control_enable_headset_button (NULL);
 }
 
@@ -481,7 +494,7 @@ static inline void headset_control_mce_phone_watcher_init (HeadsetControl *self)
 		
 		if (dbus_g_proxy_call (mce_request_proxy, MCE_CALL_STATE_GET, NULL, G_TYPE_INVALID, G_TYPE_STRING, &call_state, G_TYPE_STRING, &emergency_call_state, G_TYPE_INVALID))
 		{
-			self->priv->in_call = IS_IN_CALL;
+			self->priv->in_call = IS_IN_CALL (call_state);
 
 			g_free (call_state);
 			g_free (emergency_call_state);
@@ -492,27 +505,43 @@ static inline void headset_control_mce_phone_watcher_init (HeadsetControl *self)
 
 	self->priv->mce_signal_proxy = dbus_g_proxy_new_for_name (klass->sys_dbus_conn, MCE_SERVICE, MCE_SIGNAL_PATH, MCE_SIGNAL_IF);
 	if (!self->priv->mce_signal_proxy)
+	{
+		self->priv->in_call = FALSE;
 		return;
+	}
 
-	dbus_g_proxy_add_signal (self->priv->mce_signal_proxy, MCE_CALL_STATE_SIG, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
+	dbus_g_proxy_add_signal (self->priv->mce_signal_proxy, MCE_CALL_STATE_SIG, HEADSET_CONTROL_TYPE_2_STRINGS);
 	dbus_g_proxy_connect_signal (self->priv->mce_signal_proxy, MCE_CALL_STATE_SIG, G_CALLBACK (headset_control_call_state_changed), self, NULL);
 }
 
-static inline void headset_control_hal_proxy_init (HeadsetControl *self)
+static inline void headset_control_hal_device_proxy_init (HeadsetControl *self)
 {
 	HeadsetControlClass *klass = HEADSET_CONTROL_GET_CLASS (self);
 
 	g_assert (klass->sys_dbus_conn);
 
-	self->priv->hal_proxy = dbus_g_proxy_new_for_name (klass->sys_dbus_conn, HAL_SERVICE, HEADSET_HAL_OBJ_PATH, HAL_DEVICE_INTERFACE);
-	if (!self->priv->hal_proxy)
+	self->priv->hal_device_proxy = dbus_g_proxy_new_for_name (klass->sys_dbus_conn, HAL_SERVICE, HEADSET_HAL_OBJ_PATH, HAL_DEVICE_INTERFACE);
+	if (!self->priv->hal_device_proxy)
 	{
 		g_error ("My God! We haz no HAL proxy!");
 		g_object_unref (self);
 	}
 
 	dbus_g_object_register_marshaller (hal_marshal_VOID__STRING_STRING, G_TYPE_NONE, HEADSET_CONTROL_TYPE_2_STRINGS);
-	dbus_g_proxy_add_signal (self->priv->hal_proxy, HAL_CONDITION, HEADSET_CONTROL_TYPE_2_STRINGS);
+	dbus_g_proxy_add_signal (self->priv->hal_device_proxy, HAL_CONDITION, HEADSET_CONTROL_TYPE_2_STRINGS);
+}
+
+static inline void headset_control_hal_manager_proxy_init (HeadsetControl *self)
+{
+	HeadsetControlClass *klass = HEADSET_CONTROL_GET_CLASS (self);
+
+	g_return_if_fail (klass->sys_dbus_conn);
+
+	self->priv->hal_manager_proxy = dbus_g_proxy_new_for_name (klass->sys_dbus_conn, HAL_SERVICE, HAL_MANAGER_PATH, HAL_MANAGER_INTERFACE);
+	if (!self->priv->hal_manager_proxy)
+		return;
+
+	dbus_g_proxy_add_signal (self->priv->hal_manager_proxy, HAL_DEVICE_ADDED, G_TYPE_STRING, G_TYPE_INVALID);
 }
 
 static inline void headset_control_mafw_renderer_init (HeadsetControl *self)
@@ -617,10 +646,11 @@ static void headset_control_init (HeadsetControl *self)
 	self->priv->suspended = FALSE;
 
 	headset_control_gconf_init (self); /* A shame gconf-bridge sucks cock */
-	headset_control_hal_proxy_init (self);
+	headset_control_hal_device_proxy_init (self);
 	headset_control_mafw_renderer_init (self);
 	headset_control_playback_init (self);
 	headset_control_mce_phone_watcher_init (self);
+	headset_control_hal_manager_proxy_init (self);
 	headset_control_dbus_server_init (self);
 }
 
